#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>

#include "client-serveur.h"

int main(int argc, char *argv[]) {
  requete_t requete;
  reponse_t reponse;
  int i;
  int len;
  int fdW, fdR;
  int nbRead, nbWrite;

  if (argc != 2) {
    printf("Usage = client nomDeFamilleCherche\n");
    exit(1);
  }

  /* Prparation du champ pointAccesClient de la requte */
  sprintf(requete.pointAccesClient, "./serveur-client.%d", getpid());

  /* Prparation du champ contenu : On y recopie argv[1] en mettant en */
  /* majuscules                                                        */
  len = strlen(argv[1]);
  for (i=0 ; i < len ; i++) {
    requete.contenu[i] = toupper(argv[1][i]);
  }
  requete.contenu[len] = '\0';

  /* Cration du tube nomm du client */
  if (mkfifo(requete.pointAccesClient, S_IRUSR|S_IWUSR) < 0) {
    if (errno != EEXIST) {
      perror("mkfifo(tube nomm client");
      exit(1);
    }
    else {
      printf("%s existe deja : on suppose que c'est un pipe nomme\n",
	     requete.pointAccesClient );
      printf("et qu'on peut continuer le programme sans probleme\n");
      puts("");
    }
  }

  /* Envoi de la requte vers le serveur */
  fdW = open(NOMPOINTACCESSERV, O_WRONLY);
  if (fdW == -1) {
    perror("open(NOMPOINTACCESSERV)");
    exit(-1);
  }
  nbWrite = write(fdW, &requete, sizeof(requete));
  if (nbWrite < sizeof(requete)) {
	perror("pb ecriture sur pipe nomme");
  }

  /* On n'aura pas d'autres messages  envoyer au serveur. On peut donc    */
  /* fermer le tube                                                        */
  close(fdW);

  /* On ouvre seulement maintenant le tube pour couter la rponse, sinon  */
  /* on serait bloqu jusqu' ce que le serveur ouvre le pipe en criture  */
  /* pour envoyer sa rponse. Si on ouvrait le tube avant d'envoyer la     */
  /* requte, le serveur ne recevrait jamais la requte et donc n'ouvrirait*/
  /* jamais le tube ==> On serait bloqu indfiniment.                     */
  fdR = open(requete.pointAccesClient, O_RDONLY);
  if (fdR == -1) {
    perror("open(tube nomm)");
    exit(-1);
  }

  /* On lit la rponse */
  nbRead = read(fdR, &reponse, sizeof(reponse));
  if (nbRead != sizeof(reponse)) {
    printf("Communication avec le serveur probablement rompue\n");
    exit(1);
  }

  /* On affiche la rponse */
  printf("Reponse du serveur = \"%s\"\n", reponse.contenu);

  /* NB : Dans cette application, on utilise pas le point d'accs priv */
  /* vers le serveur                                                    */

  /* On ferme le tube ct rception */
  close(fdR);

  /* On dtruit le tube nomm du client (puisque ce tube nomm ne servira */
  /* plus jamais.                                                         */
  if (unlink(requete.pointAccesClient) < 0) {
    perror("unlink");
    exit(1);
  }

  return 0;
}


   
