#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "client-serveur.h"

#define NBENFANTS 26

int pereEnfant[NBENFANTS][2];

void traiterRequete(requete_t *requete, FILE *f, int debutRech, int finRech){
  reponse_t reponse;
  char ligne[256];
  int fdW;
  int nbWrite;

  /* Calcul reponse->pointAccesPrive */
  strcpy(reponse.pointAccesPrive, "");
              /* Le point d'accs priv n'est pas utilis dans cette appli */

  /* Calcul reponse.contenu. */
  if (fseek(f, debutRech, SEEK_SET) < 0) {
    perror("fseek");
    exit(EXIT_FAILURE);
  }
  while ((feof(f) == 0) && (ftell(f) < finRech)) {
    if (fgets(ligne, sizeof(ligne), f) != NULL) {
      if (strncmp(ligne, requete->contenu, strlen(requete->contenu)) == 0){
	break;
      }
    }
  }
  if ((feof(f) == 0) && (ftell(f) < finRech)) {
    strcpy(reponse.contenu, ligne);
    reponse.contenu[strlen(ligne)-1] = '\0'; /* Supprime le '\n' final */
  }
  else {
    strcpy(reponse.contenu,"Dsol, je ne le connais pas");
  }

  /* Affichage */
  printf("L'enfant %d du serveur a recu \"%s\" et repond \"%s\"\n",
	 getpid(),
	 requete->contenu,
	 reponse.contenu );

  /* Connexion au point d'accs client et rponse */
  fdW = open(requete->pointAccesClient, O_WRONLY);
  if (fdW == -1) {
    perror("open(pointAccessClient)");
    exit(EXIT_FAILURE);
  }
  nbWrite = write(fdW, &reponse, sizeof(reponse));
  if (nbWrite < sizeof(reponse)) {
    perror("pb ecriture sur pipe nomme");
  }

  /* Dans cette application, le client ne renvoie pas de requte ultrieure*/
  /* ncessitant une rponse ==> On peut fermer ce tube                    */
  close(fdW);

}

void attenteRequete(int monNum){
  requete_t requete;
  FILE *f;
  int debutRech;
  int finRech;
  char ligne[256];

  /* Ouverture du fichier de M. et Mme */
  f = fopen("mEtMme.txt", "r");
  if (f == NULL) {
    perror("open(mEtMme.txt)");
    exit(EXIT_FAILURE);
  }

  /* On cherche a partir d'ou il y a des enregistrements qui correspondent a */
  /* la lettre associe a monNum (si monNum == 0, 'A', 1 'B'...              */
  while (feof(f) == 0) {
    debutRech = ftell(f);
    if (fgets(ligne, sizeof(ligne), f) != NULL) {
      if ((ligne[0]-'A') == monNum) {
	break;
      }
    }
  }
  if (feof(f) == 0) {
    /* On a trouv la lettre dans le fichier : on cherche le dernier endroit*/
    /* o elle apparat pour avoir finRech                                  */
    int dernFgetsOK;
    while (feof(f) == 0) {
      dernFgetsOK = ftell(f);
      if (fgets(ligne, sizeof(ligne), f) != NULL) {
	if ((ligne[0]-'A') != monNum) {
	  finRech = ftell(f);
	  break;
	}
      }
    }
    if (feof(f) != 0){
      finRech = dernFgetsOK + strlen(ligne);
    }
  }
  else {
    /* On n'a pas trouv la lettre dans le fichier : on positionne debutRech */
    /* et finRech de la manire qui nous arrange le plus pour la suite       */
    debutRech = 0;
    finRech = 0;
  }

  /* Attente de requte */
  while(1){

    /* On attend une requte */
    read(pereEnfant[monNum][0], &requete, sizeof(requete));

    /* On la traite */
    traiterRequete(&requete, f, debutRech, finRech);
  }
}
    
int main() {
  requete_t requete;
  int fdR;
  int nbRead;
  int i;

  /* Cration du tube nomm du serveur */
  if (mkfifo(NOMPOINTACCESSERV, S_IRUSR|S_IWUSR) < 0) {
    if (errno != EEXIST) {
      perror("mkfifo(tube nomm client");
      exit(EXIT_FAILURE);
    }
    else {
      printf("%s existe deja : on suppose que c'est un pipe nomme\n",
	     NOMPOINTACCESSERV );
      printf("et qu'on peut continuer le programme sans probleme\n");
      puts("");
    }
  }

  /* Cration du pool de fils                                             */
  /* Le pool est reprsent par un tube sur lequel le pre lit les        */
  /* numros des fils qui se considrent prts  traiter des requtes     */
  for (i=0 ; i<NBENFANTS ; i++){
    pipe(pereEnfant[i]);
  }
  for (i=0 ; i<NBENFANTS ; i++){
    if (fork() == 0) {
      attenteRequete(i);
    }
  }

  /* Ouverture de ce tube nomm (equivalent a une ouverture de connexion).
     NB : On l'ouvre en RDWR et pas en RDONLY,
     car (cf. Blaess) : si nous ouvrons le client en lecture seule,  chaque
     fois que le client se termine, la lecture du tube nomme va echouer
     car le noyau detecte une fermeture de fichier. En demandant un tube
     en lecture et en ecriture, nous evitons ce genre de situation, car il
     reste toujours au moins un descripteur ouvert sur la sortie
  */
  fdR = open(NOMPOINTACCESSERV, O_RDWR);
  if (fdR == -1) {
    perror("open(tube nomm)");
    exit(EXIT_FAILURE);
  }
  /* Attente de requtes */
  do{
    nbRead = read(fdR, &requete, sizeof(requete));
    if (nbRead != sizeof(requete)) {
      perror("read sur tube nomme NOK");
      return EXIT_FAILURE;
    }
    /* On envoie la requte a l'enfant concern */
    if ((requete.contenu[0] >='A') && (requete.contenu[0] <= 'Z')) {
      write(pereEnfant[requete.contenu[0]-'A'][1], &requete, sizeof(requete));
    }
    else {
      /* On demande  l'enfant qui s'occupe des 'A' de rpondre "Dsol" */
      /* pour nous                                                       */
      write(pereEnfant[0][1], &requete, sizeof(requete));
    }
  }while(1);
}


   
